# Copyright (c) 2018 NTT DATA
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf.urls import url

from masakaridashboard.hosts import views


HOST = r'^(?P<host_id>[^/]+)/%s$'
urlpatterns = [
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(HOST % 'detail', views.DetailView.as_view(), name='detail'),
    url(HOST % 'update', views.UpdateView.as_view(), name='update'),
]
